use axum::{extract::Path, response::IntoResponse, Extension, Json};
use axum_extra::extract::CookieJar;
use tetratto_core::model::{ApiReturn, Error};
use crate::{get_user_from_token, State};

pub async fn update_last_message_request(
    jar: CookieJar,
    Extension(data): Extension<State>,
    Json((channel_id, message_id)): Json<(usize, usize)>,
) -> impl IntoResponse {
    let data = &(data.read().await).0;
    let mut user = match get_user_from_token!(jar, data) {
        Some(ua) => ua,
        None => return Json(Error::NotAllowed.into()),
    };

    user.subscriptions.insert(channel_id, message_id);

    if let Err(e) = data
        .update_user_subscriptions(user.id, user.subscriptions)
        .await
    {
        return Json(e.into());
    }

    Json(ApiReturn {
        ok: true,
        message: "Subscription connection".to_string(),
        payload: (),
    })
}

pub async fn delete_request(
    jar: CookieJar,
    Extension(data): Extension<State>,
    Path(channel): Path<usize>,
) -> impl IntoResponse {
    let data = &(data.read().await).0;
    let mut user = match get_user_from_token!(jar, data) {
        Some(ua) => ua,
        None => return Json(Error::NotAllowed.into()),
    };

    user.subscriptions.remove(&channel);

    if let Err(e) = data
        .update_user_subscriptions(user.id, user.subscriptions)
        .await
    {
        return Json(e.into());
    }

    Json(ApiReturn {
        ok: true,
        message: "Subscription removed".to_string(),
        payload: (),
    })
}
